/*
 * Portions of this software are copyrighted by the 
 * 
 *     University of South Carolina,
 *     Sun Microsystems, Inc., 
 * 
 * and other parties. 
 * 
 * IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
 * DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE
 * IS PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE
 * NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 */


package edu.sc.seis.TauP;

import java.io.*;

/** Utility class to keep track of criticalpoints (discontinuities or
 *  reversals in slowness gradient) within slowness and velocity models.
 *
 * @version 1.0  Tue Feb  2 20:48:34 GMT 1999
 * @author H. Philip Crotwell
 */
public class CriticalDepth implements Cloneable, Serializable {

      /** depth in kilometers at which there is a critical point. */
   public double depth;

      /** layer number within the velocity model with this depth at its top. */
   public int velLayerNum;

      /** slowness layer for P waves with this depth at its top. This can be PLayers.size() for
       *  the last critical layer. */
   public int PLayerNum;

      /** slowness layer for S waves with this depth at its top. This can be SLayers.size() for
       *  the last critical layer. */
   public int SLayerNum;

// Constructors

   public CriticalDepth() {
   }

   public CriticalDepth(double depth, int velLayerNum, int PLayerNum, 
   int SLayerNum) {
      this.depth = depth;
      this.velLayerNum = velLayerNum;
      this.PLayerNum = PLayerNum;
      this.SLayerNum = SLayerNum;
   }

// Accessor methods

   public void setVelLayerNum(int layerNum) {
      velLayerNum = layerNum;
   }

   public void setPLayerNum(int layerNum) {
      PLayerNum = layerNum;
   }

   public void setSLayerNum(int layerNum) {
      SLayerNum = layerNum;
   }

   public int getVelLayerNum() {
      return velLayerNum;
   }

   public int getPLayerNum() {
      return PLayerNum;
   }

   public int getSLayerNum() {
      return SLayerNum;
   }

      /** sets slowness layer for waveType waves with this depth at its top.
       */
   public void setLayerNum(int layerNum, boolean isPWave) {
      if (isPWave) {
         PLayerNum = layerNum;
      } else {
         SLayerNum = layerNum;
      }
   }

      /** @returns slowness layer for waveType waves with this depth at its top.
       */
   public int getLayerNum(boolean isPWave) {
      if (isPWave) {
         return PLayerNum;
      } else {
         return SLayerNum;
      }
   }
   
   
   public Object clone() {
      CriticalDepth newObject;
      try {
         newObject = (CriticalDepth)super.clone();
         return newObject;
      } catch (CloneNotSupportedException e) {
         // Can't happen, but...
         System.err.println("Caught CloneNotSupportedException: "+
            e.getMessage());
         throw new InternalError(e.toString());
      }
   }
}
